﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Runtime.InteropServices;

namespace Generator.Application.Board
{
    public sealed class HaskellRuntimeSingleton
    {
        #region Singleton
        private static readonly Lazy<HaskellRuntimeSingleton> lazy = new(() => new HaskellRuntimeSingleton());     
        public static HaskellRuntimeSingleton Instance { get { return lazy.Value; } }

        #endregion

        #region DLLImports
        [DllImport("haskellInterface.dll", CallingConvention = CallingConvention.Cdecl)]
        private static extern void hs_init(IntPtr argc, IntPtr argv);

        [DllImport("HaskellInterface.dll", CallingConvention = CallingConvention.Cdecl)]
        private static extern void hs_exit();

        [DllImport("HaskellInterface.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern IntPtr generateEasyBoard(int height, int width, int seed, double distribution);

        [DllImport("HaskellInterface.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern IntPtr generateMediumBoard(int height, int width, int seed, double distribution);

        [DllImport("HaskellInterface.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern void freeSBoard(IntPtr ptr);

        [DllImport("HaskellInterface.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern IntPtr getSolvedSCellPtrByIndex(IntPtr ptr, int row, int col);

        [DllImport("HaskellInterface.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern IntPtr getUnsolvedSCellPtrByIndex(IntPtr ptr, int row, int col);
        #endregion


        private HaskellRuntimeSingleton()
        {
            Console.WriteLine("Initializing runtime...");
            hs_init(IntPtr.Zero, IntPtr.Zero);
        }

        ~HaskellRuntimeSingleton()
        {
            Console.WriteLine("Exiting runtime...");
            hs_exit();
        }

    }
}
