module Board.Cell(
    Cell(..),
    Coordinate(..),
    CellType(..),
    cellIsGrey,
    cellIsWhite,
    cellIsBlack,
    printCell
)where
import Data.Maybe (fromMaybe)

type Coordinate = (Int, Int)

data Cell = Cell {position :: Coordinate,
    number :: Maybe Int,
    cellType :: CellType} deriving(Eq)

instance Show Cell where
    show Cell{position = p, number = n, cellType = t} = case t of
      Grey -> show p ++ ": " ++ show t ++ " " ++ show (fromMaybe 0 n) ++ "  "
      _ -> show p ++ ": " ++ show t ++ " " ++ show (fromMaybe 0 n) ++ " "


data CellType = White | Grey | Black
    deriving(Show, Eq)

cellIsGrey :: Cell -> Bool
cellIsGrey cell = cellType cell == Grey

cellIsBlack :: Cell -> Bool 
cellIsBlack cell = cellType cell == Black

cellIsWhite :: Cell -> Bool 
cellIsWhite cell = cellType cell == White

printCell :: Cell -> IO()
printCell cell = 
    let
        num = number cell
        t = cellType cell
        pos = position cell
        str = case t of
            Grey -> show t ++ " "
            _ -> show t
    in
        case num of
            Nothing -> putStr $ str ++ " "
            Just x -> putStr $ "  " ++ show x ++ "   " 