module Board.Modifier(
    blacken,
    whiten,
    markCells,
    placeNumber,
    removeAllMarkings,
    unMarkCells
)where

import Data.Maybe(isNothing)

import Board.Board(
    Board(..),
    coordinateIsValid
    )

import Board.Cell(
    Cell(..),
    CellType(..),
    Coordinate,
    cellIsGrey
    )

blacken :: Board -> [Coordinate] -> Board
blacken = markCells Black

whiten :: Board -> [Coordinate] -> Board 
whiten = markCells White

markCells :: CellType -> Board -> [Coordinate] -> Board
markCells _ [] _ = []
markCells _ board [] = board
markCells t board (head:tail) = let
        newBoard = markCell t board head
    in
        markCells t newBoard tail

markCell :: CellType -> Board -> Coordinate -> Board
markCell _ [] _ = []
markCell Grey board _ = board
markCell newCellType board position
    | not (coordinateIsValid board position) = board
    | otherwise = topRows ++ newRow:bottomRows
    where
        r = fst position
        c = snd position
        (topRows, oldRow:bottomRows) = splitAt r board
        (leftCells, cell:rightCells) = splitAt c oldRow
        newRow = leftCells ++ Cell { position=(r, c), number=number cell, cellType=newCellType}:rightCells

placeNumber :: Board -> Int -> Coordinate -> Board
placeNumber [] _ _ = []
placeNumber board num pos
    | not (coordinateIsValid board pos) = board
    | otherwise =
        _insertCell board Cell{position=pos, cellType=White, number = Just num}

_insertCell :: Board -> Cell -> Board
_insertCell [] _ = []
_insertCell board newcell
    | not (coordinateIsValid board (position newcell)) = board
    | otherwise =
        topRows ++ newRow:bottomRows
    where
        r = fst $ position newcell
        c = snd $ position newcell
        (topRows, oldRow:bottomRows) = splitAt r board
        (leftCells, oldcell:rightCells) = splitAt c oldRow
        newRow = leftCells ++ newcell:rightCells


removeAllMarkings :: Board -> Board
removeAllMarkings [] = []
removeAllMarkings board =
    let
        cells = concat board
        unNumberedCells = [cell | cell <- cells, isNothing (number cell)]
    in
        unMarkCells board (map position unNumberedCells)

unMarkCells :: Board -> [Coordinate] -> Board
unMarkCells [] _ = []
unMarkCells board [] = board
unMarkCells board (head:tail) =
    let
        newBoard = _unMarkCell board head
    in
        unMarkCells newBoard tail

_unMarkCell :: Board -> Coordinate -> Board
_unMarkCell [] _ = []
_unMarkCell board coordinate
    |not (coordinateIsValid board coordinate) = board
    |otherwise =
        let
            r = fst coordinate
            c = snd coordinate
            (topRows, oldRow:bottomRows) = splitAt r board
            (leftCells, oldcell:rightCells) = splitAt c oldRow
            newRow = leftCells ++ Cell {position=(r, c), number=number oldcell, cellType=Grey}:rightCells
        in
            topRows ++ newRow:bottomRows