module Board.Monitor(
    Monitor(..),
    (>\\=),
    (>\=),
    extract
)
where

class Modifiable m where
    (>\=) :: m a -> (a -> m a) -> m a
    (>\\=) :: m a -> (a -> m a) -> m a
    extract :: m a -> a

data Monitor a = Changed a | Unchanged a deriving(Show)

instance Modifiable Monitor where
    -- Only allows Unchanged variables to be modified
    (>\=) (Changed x) _ = Changed x
    (>\=) (Unchanged x) f = f x

    -- Allows Changed variables to be modified, but if input value is Changed, output will also be Changed, regardless of it being modified
    (>\\=) (Changed x) f = Changed $ extract (f x)
    (>\\=) (Unchanged x) f = f x

    extract (Changed a) = a
    extract (Unchanged a) = a

