module Region.Island(
    Island,
    findAllIslands,
    isIslandNumbered,
    isIslandComplete,
    findTotalNumberedIslandSize,
    findNumberedIslands,
    findAllCommonBorderCells,
    islandGrowthPotential,
    findLargestIsland
)where

import Board.Cell(Cell(..), cellIsWhite)
import Board.Board(Board)

import Region.Region(Region,findAreaNeighbours, discoverFullRegion)

import Data.Maybe ( isJust, mapMaybe )

type Island = Region

isIslandNumbered :: Island -> Bool
isIslandNumbered [] = False
isIslandNumbered island = case _findIslandSize island of
    Nothing -> False
    _ -> True

isIslandComplete :: Island -> Bool
isIslandComplete [] = False
isIslandComplete island = islandGrowthPotential island == 0

islandGrowthPotential :: Island -> Int
islandGrowthPotential [] = 0
islandGrowthPotential island = case _findIslandSize island of
  Nothing -> (-1)
  Just n -> n - length island

_findIslandSize :: Island -> Maybe Int
_findIslandSize [] = Nothing
_findIslandSize (head:tail) = case number head of
  Nothing -> _findIslandSize tail
  Just n -> Just n


findTotalNumberedIslandSize :: Board -> Int
findTotalNumberedIslandSize [] = 0
findTotalNumberedIslandSize board = let
        numberedCells = _findNumberedCells board
    in
        sum $ mapMaybe number numberedCells


findNumberedIslands :: Board -> [Island]
findNumberedIslands [] = []
findNumberedIslands board =
    [island | island <- findAllIslands board,
        isIslandNumbered island]

findAllIslands :: Board -> [Island]
findAllIslands [] = []
findAllIslands board = _groupIslands board (_findIslandCells board)

_groupIslands :: Board -> [Cell] -> [Island]
_groupIslands [] _ = []
_groupIslands _ [] = []
_groupIslands board (head:tail) = let
        headIsland = discoverFullRegion board [head]
    in
        headIsland : _groupIslands board [c | c <- tail, c `notElem` headIsland]


_findIslandCells :: Board -> [Cell]
_findIslandCells [] = []
_findIslandCells board = concatMap _findIslandCellsInRow board

_findIslandCellsInRow :: [Cell] -> [Cell]
_findIslandCellsInRow [] = []
_findIslandCellsInRow row = [cell | cell <- row, cellIsWhite cell]


_findNumberedCells :: Board -> [Cell]
_findNumberedCells [] = []
_findNumberedCells board = concatMap _findNumberedCellsInRow board

_findNumberedCellsInRow :: [Cell] -> [Cell]
_findNumberedCellsInRow [] = []
_findNumberedCellsInRow row = [cell | cell <- row, cellIsWhite cell, isJust (number cell)]


findAllCommonBorderCells :: Board -> [Island] -> [Cell]
findAllCommonBorderCells [] _ = []
findAllCommonBorderCells _ [] = []
findAllCommonBorderCells board (head:tail) = _findNonUniqueBorderCells board head tail ++ findAllCommonBorderCells board tail

_findNonUniqueBorderCells :: Board -> Island -> [Island] -> [Cell]
_findNonUniqueBorderCells [] _ _ = []
_findNonUniqueBorderCells _ [] _ = []
_findNonUniqueBorderCells _ _ [] = []
_findNonUniqueBorderCells board island (head:tail) = _findCommonBorderCells board island head ++ _findNonUniqueBorderCells board island tail

_findCommonBorderCells :: Board -> Island -> Island -> [Cell]
_findCommonBorderCells [] _ _ = []
_findCommonBorderCells _ [] _ = []
_findCommonBorderCells _ _ [] = []
_findCommonBorderCells board island1 island2 = let
        border1 = findAreaNeighbours board (map position island1)
        border2 = findAreaNeighbours board (map position island2)
    in
        [c | c <- border1, c `elem` border2]


findLargestIsland :: Board -> Island
findLargestIsland [] = []
findLargestIsland board =
    let
        islands = findAllIslands board
    in
        _getLongest islands

_getLongest :: [Island] -> Island
_getLongest [] = []
_getLongest (head:tail) =
    let
        tailMax = _getLongest tail
    in
        if length head > length tailMax then
            head
        else
            tailMax