module Region.Region(
    Region,
    findAreaNeighbours,
    isCellPotentialPool,
    isCellPotentialDryLand,
    findAllGreyCells,
    discoverFullRegion
)where

import Board.Cell(Cell(..),CellType(Black, White),Coordinate, cellIsGrey)
import Board.Board(Board,coordinateIsValid)

type Region = [Cell]

_findValidNeighboursOfCoordinate :: Board -> Coordinate -> [Cell]
_findValidNeighboursOfCoordinate [] _ = []
_findValidNeighboursOfCoordinate board pos
    | not (coordinateIsValid board pos) = []
    | otherwise = [board !! i !! j |
        i <- [(r-1)..(r+1)],
        j <- [(c-1)..(c+1)],
        coordinateIsValid board (i, j),
        abs (i-r) /= abs (j-c)]
    where 
        height = length board
        width = length (head board)
        r = fst pos
        c = snd pos
                        
findAreaNeighbours :: Board -> [Coordinate] -> [Cell]
findAreaNeighbours [] _ = []
findAreaNeighbours _ [] = []
findAreaNeighbours board all@(x:xs) =
    let headNeighbours = _findValidNeighboursOfCoordinate board x
        tailNeighbours = [b | b <- findAreaNeighbours board xs,
            b `notElem` headNeighbours]

    in [a | a <- headNeighbours ++ tailNeighbours,
        position a `notElem` all,
        coordinateIsValid board (position a)]


discoverFullRegion :: Board -> Region -> Region
discoverFullRegion [] _ = []
discoverFullRegion _ [] = []
discoverFullRegion board region
    | null sameTypeNeighbours = region
    | otherwise = discoverFullRegion board (region++sameTypeNeighbours)
    where
        sameTypeNeighbours = [c | c <- findAreaNeighbours board (map position region), cellType c == cellType (head region)]


isCellPotentialPool :: Board -> Coordinate -> Bool 
isCellPotentialPool [] _ = False
isCellPotentialPool board pos
    | not $ coordinateIsValid board pos = False 
    | otherwise = let
            row = fst pos
            col = snd pos
            neighbours = _findValidNeighboursOfCoordinate board pos
            corners = [board !! i !! j | i <- [row - 1, row + 1],
                j <- [col - 1, col + 1],
                coordinateIsValid board (i, j)]
            cornerLs = [corner:[c | c <- _findValidNeighboursOfCoordinate board $ position corner,
                    c `elem` neighbours]
                | corner <- corners]
            poolCandidates = [corner | corner <- cornerLs,
                _getRegionTypeIfUniform corner == Just Black]
        in
            not $ null poolCandidates
        

isCellPotentialDryLand :: Board -> Coordinate -> Bool
isCellPotentialDryLand [] _ = False 
isCellPotentialDryLand board pos
    | not $ coordinateIsValid board pos = False 
    | otherwise = 
        let
            row = fst pos
            col = snd pos
            neighbours = _findValidNeighboursOfCoordinate board pos
            corners = [board !! i !! j | i <- [row - 1, row + 1],
                j <- [col - 1, col + 1],
                coordinateIsValid board (i, j)]
            cornerLs = [corner:[c | c <- _findValidNeighboursOfCoordinate board $ position corner,
                    c `elem` neighbours]
                | corner <- corners]
            dryLandCandidates = [corner | corner <- cornerLs,
                _getRegionTypeIfUniform corner == Just White]
        in
            not $ null dryLandCandidates

_getRegionTypeIfUniform :: Region -> Maybe CellType 
_getRegionTypeIfUniform [] = Nothing
_getRegionTypeIfUniform [x] = Just $ cellType x
_getRegionTypeIfUniform (head:tail) = 
    let
        headType = cellType head 
        tailType = _getRegionTypeIfUniform tail 
    in
        case tailType of
            Nothing -> Nothing
            Just x -> if headType == x then
                    Just headType
                else
                    Nothing

findAllGreyCells :: Board -> [Cell]
findAllGreyCells [] = []
findAllGreyCells board = concatMap _findGreyCellsInRow board

_findGreyCellsInRow :: [Cell] -> [Cell]
_findGreyCellsInRow [] = []
_findGreyCellsInRow row = [cell | cell <- row, cellIsGrey cell]
