module Region.Water(
    findIncompleteWaterRegions,
    findAllWaterRegions,
)where

import Board.Cell(Cell, cellIsBlack)
import Board.Board(Board, numCellsInBoard)

import Region.Region(Region, discoverFullRegion)
import Region.Island(findTotalNumberedIslandSize)


findIncompleteWaterRegions :: Board -> [Region]
findIncompleteWaterRegions [] = []
findIncompleteWaterRegions board =
    let
        waterRegions = findAllWaterRegions board
    in case waterRegions of
      [] -> []
      [x] -> [x | not (_isWaterRegionComplete board x)]
      (x:xs) -> x:xs

_isWaterRegionComplete :: Board -> Region -> Bool
_isWaterRegionComplete [] _ = False
_isWaterRegionComplete _ [] = False
_isWaterRegionComplete board region = length region == numCellsInBoard board - findTotalNumberedIslandSize board


findAllWaterRegions :: Board -> [Region]
findAllWaterRegions [] = []
findAllWaterRegions board = _groupWaterCells board (_findWaterCells board)

_groupWaterCells :: Board -> [Cell] -> [Region]
_groupWaterCells [] _ = []
_groupWaterCells _ [] = []
_groupWaterCells board (head:tail) = let
        headRegion = discoverFullRegion board [head]
    in
        headRegion : _groupWaterCells board [r | r <- tail, r `notElem` headRegion]


_findWaterCells :: Board -> [Cell]
_findWaterCells [] = []
_findWaterCells board = concatMap _findWaterCellsInRow board

_findWaterCellsInRow :: [Cell] -> [Cell]
_findWaterCellsInRow [] = []
_findWaterCellsInRow row = [cell | cell <- row, cellIsBlack cell]