module Techniques.Complete(
    complete
)where

import Board.Cell(Cell(position), cellIsGrey)
import Board.Board(Board)
import Board.Monitor(Monitor(..), (>\\=), (>\=))
import Board.Modifier(blacken)

import Region.Region(findAreaNeighbours)
import Region.Island(Island, findAllIslands, isIslandComplete)

complete :: Board -> Monitor Board
complete [] = Unchanged []
complete board = _completeIslands (findAllIslands board) board

_completeIslands :: [Island] -> Board -> Monitor Board
_completeIslands _ [] = Unchanged [] -- Unncessary?
_completeIslands [] board = Unchanged board
_completeIslands (head:tail) board = let
        modifiedBoard = 
            if isIslandComplete head then
                _completeIsland head board
            else
                Unchanged board
    in
        modifiedBoard >\\= _completeIslands tail

_completeIsland :: Island -> Board -> Monitor Board
_completeIsland _ [] = Unchanged []
_completeIsland [] board = Unchanged board
_completeIsland island board = let
        islandCoords = map position island
        border = findAreaNeighbours board islandCoords
        greyBorderCoords = [position c | c <- border, cellIsGrey c]
        modified = not $ null greyBorderCoords
    in
        if modified then
            Changed (blacken board greyBorderCoords)
        else
            Unchanged board