module Techniques.Expand(
    expand
)where

import Board.Cell(Cell(..), cellIsGrey, cellIsWhite)
import Board.Board(Board)
import Board.Monitor(Monitor(..), (>\\=))
import Board.Modifier(markCells)

import Region.Region(Region, findAreaNeighbours, discoverFullRegion)
import Region.Incomplete(findAllIncompleteRegions)

expand :: Board -> Monitor Board 
expand [] = Unchanged []
expand board = 
    let
        incompleteRegions = findAllIncompleteRegions board
        incompleteIslands = [region | region <- incompleteRegions, cellIsWhite (head region)]
        incompleteWaterRegions = [region | region <- incompleteRegions, region `notElem` incompleteIslands]
        modifiedBoard = _expandRegions incompleteIslands board -- Ensure all islands are expanded maximally before the water region is. Might be unnecessary
    in
        case modifiedBoard of
            Changed x -> modifiedBoard >\\= expand
            Unchanged x -> modifiedBoard >\\= _expandRegions incompleteWaterRegions


_expandRegions :: [Region] -> Board -> Monitor Board 
_expandRegions _ [] = Unchanged []
_expandRegions [] board = Unchanged board
_expandRegions (head:tail) board = let
        modifiedBoard = _expandRegion head board
    in 
        modifiedBoard >\\= _expandRegions tail

_expandRegion :: Region -> Board -> Monitor Board 
_expandRegion _ [] = Unchanged []
_expandRegion [] board = Unchanged board
_expandRegion region board = let
        t = cellType (head region)
        fullRegion = discoverFullRegion board region
        unknownBorder = [cell | cell <- findAreaNeighbours board (map position fullRegion),
            cellIsGrey cell]
    in
        case unknownBorder of
            [x] -> Changed (markCells t board [position x]) 
            _ -> Unchanged board
