module Techniques.Surround(
    surround
)where

import Board.Cell (CellType, Cell(..))
import Board.Board(Board)
import Board.Monitor(Monitor(..), (>\\=))
import Region.Region (findAllGreyCells, findAreaNeighbours)
import Board.Modifier (markCells)
import Debug.Trace

surround :: Board -> Monitor Board
surround [] = Unchanged []
surround board =
    let
        greyCells = findAllGreyCells board
    in
        _checkCellSurroundings greyCells board

_checkCellSurroundings :: [Cell] -> Board -> Monitor Board
_checkCellSurroundings _ [] = Unchanged []
_checkCellSurroundings [] board = Unchanged board
_checkCellSurroundings (head:tail) board =
    let
        modifiedBoard = _checkCellSurrounding head board
    in
        modifiedBoard >\\= _checkCellSurroundings tail

_checkCellSurrounding :: Cell -> Board -> Monitor Board
_checkCellSurrounding _ [] = Unchanged []
_checkCellSurrounding cell board =
    let
        neighbours = findAreaNeighbours board [position cell]
    in
        case _typeOfCells neighbours of
            Nothing -> Unchanged board
            Just t ->  if t /= cellType cell then
                Changed $ markCells t board [position cell]
            else
                Unchanged board

_typeOfCells :: [Cell] -> Maybe CellType
_typeOfCells [] = Nothing
_typeOfCells [cell] = Just $ cellType cell
_typeOfCells (head:tail) =
    let
        headType = cellType head
        tailType = _typeOfCells tail
    in case tailType of
        Nothing -> Nothing
        Just t -> if t == headType then
                Just t
            else
                Nothing
