module Techniques.Unreachable(
    unreachable
)where

import Board.Cell(Cell(..), cellIsBlack, cellIsWhite)
import Board.Board(Board)
import Board.Monitor(Monitor(..), (>\\=))
import Board.Modifier (blacken)

import Region.Region(findAllGreyCells, findAreaNeighbours, discoverFullRegion)
import Region.Island(Island, islandGrowthPotential, isIslandNumbered)

unreachable :: Board -> Monitor Board
unreachable [] = Unchanged []
unreachable board =
    let
        greyCells = findAllGreyCells board
    in
        _markUnreachableCells greyCells board


_markUnreachableCells :: [Cell] -> Board -> Monitor Board
_markUnreachableCells _ [] = Unchanged []
_markUnreachableCells [] board = Unchanged board
_markUnreachableCells (head:tail) board =
    let
        modifiedBoard = _markCellIfUnreachable head board
    in
        modifiedBoard >\\= _markUnreachableCells tail

_markCellIfUnreachable :: Cell -> Board -> Monitor Board
_markCellIfUnreachable _ [] = Unchanged[]
_markCellIfUnreachable cell board =
    if not (_isCellReachable board cell) then
        Changed $ blacken board [position cell]
    else
        Unchanged board

_isCellReachable :: Board -> Cell -> Bool
_isCellReachable [] _ = False
_isCellReachable board cell = _isCellReachableFromArea board [cell] 1

_isCellReachableFromArea :: Board -> [Cell] -> Int -> Bool
_isCellReachableFromArea [] _ _ = False
_isCellReachableFromArea board [] _ = False
_isCellReachableFromArea board area distance =
    let
        neighbours = [cell | cell <- findAreaNeighbours board (map position area),
            not $ cellIsBlack cell]
        whiteNeighbours = [cell | cell <- neighbours, cellIsWhite cell]
        greyNeighbours = [cell | cell <- neighbours, cell `notElem` whiteNeighbours]
        islands = _findFullIslands board whiteNeighbours
        numberedIslands = [island | island <- islands, isIslandNumbered island]
        unnumberedIslandCells = [cell | cell <- whiteNeighbours, cell `notElem` concat numberedIslands]
        expandableNeighbours = greyNeighbours ++ unnumberedIslandCells
    in
        _anyIslandReachable board numberedIslands distance || (
        not (null expandableNeighbours) &&
        _isCellReachableFromArea board (area++expandableNeighbours) (distance+1))

_findFullIslands :: Board -> [Cell] -> [Island]
_findFullIslands [] _ = []
_findFullIslands board [] = []
_findFullIslands board (head:tail) =
    let
        headIsland = discoverFullRegion board [head]
        remainderCells = [cell | cell <- tail, cell `notElem` headIsland]
    in
        headIsland:_findFullIslands board remainderCells

_anyIslandReachable :: Board -> [Island] -> Int -> Bool
_anyIslandReachable [] _ _= False
_anyIslandReachable _ [] _= False
_anyIslandReachable board (head:tail) distance =
        (islandGrowthPotential head >= distance) ||
        _anyIslandReachable board tail distance

