module Test.Generator(
    testPoolCheck,
    testRandomBoardGenerator,
    testSeededBoardGenerator
)where

import System.Random (newStdGen, mkStdGen)

import Board.Board(printBoard)
import Board.Generator(generateGreyBoard, generateAndCheckRandomCompletedBoard)
import Board.Modifier(blacken)

import Region.Region(isCellPotentialPool)

testPoolCheck :: IO()
testPoolCheck =
    let
        baseBoard = generateGreyBoard (5, 5)
        markedBoard = blacken baseBoard [(0,0), (0,3), (1,1), (1,3), (1,4), (2,0), (2,2), (2,3), (3,1), (3,3), (3,4), (4,0), (4,1), (4,3)]
    in do
        printBoard baseBoard
        putStrLn ""
        printBoard markedBoard
        putStrLn ""
        putStrLn $ "(0, 2): " ++ show (isCellPotentialPool markedBoard (0,2))
        putStrLn $ "(0, 4): " ++ show (isCellPotentialPool markedBoard (0,4))
        putStrLn $ "(1, 0): " ++ show (isCellPotentialPool markedBoard (1,0))
        putStrLn $ "(2, 4): " ++ show (isCellPotentialPool markedBoard (2,4))
        putStrLn $ "(3, 0): " ++ show (isCellPotentialPool markedBoard (3,0))
        putStrLn $ "(3, 2): " ++ show (isCellPotentialPool markedBoard (3,2))
        putStrLn $ "(4, 2): " ++ show (isCellPotentialPool markedBoard (4,2))
        putStrLn $ "(4, 4): " ++ show (isCellPotentialPool markedBoard (4,4))

testRandomBoardGenerator :: IO()
testRandomBoardGenerator = do
    gen <- newStdGen
    let
        board = generateAndCheckRandomCompletedBoard (7,7) 0.25 gen
    printBoard board

testSeededBoardGenerator :: IO()
testSeededBoardGenerator = do
    let gen = mkStdGen 2
        board = generateAndCheckRandomCompletedBoard (5,5) 0.25 gen
    printBoard board
