﻿using System;
using System.Collections.Generic;
using System.Text;
using Xamarin.Forms;
using Xamarin.CommunityToolkit.Effects;
using System.Windows.Input;

namespace MasterGame.Models.Cells
{
    public class BoxCell : BoxView
    {
        public (int Row, int Col) Position { get; set; }
        public (double X, double Y) Size { get; set; }
        public ICommand LongPressCommand { get; }
        public ICommand ShortPressCommand { get; }

        public BoxCell((int row, int col) position, (double x, double y) size) : base()
        {
            Position = position;
            Size = size;

            HeightRequest = Size.Y;
            WidthRequest = Size.X;
            CornerRadius = 0;
            BackgroundColor = Color.White;

            LongPressCommand = new Command(OnLongPressed);
            ShortPressCommand = new Command(OnTapped);

            Effects.Add(new TouchEffect());

            TouchEffect.SetLongPressCommand(this, LongPressCommand);

            TouchEffect.SetCommand(this, ShortPressCommand);

        }

        protected virtual void OnTapped()
        {
            if (BackgroundColor == Color.White)
            {
                BackgroundColor = Color.Black;
            }
            else
            {
                BackgroundColor = Color.White;
            }
        }

        protected virtual void OnLongPressed()
        {
            BackgroundColor = Color.Green;
        }
    }
}
