﻿using MasterGame.Common.Dtos;
using System;
using System.Net.Http;
using System.Text.Json;
using System.Threading.Tasks;

namespace MasterGame.Models
{
    public sealed class GeneratorClient
    {
        private readonly HttpClient _httpClient;
        private readonly Uri _uri = new Uri("");
        private readonly string _unseeded = "/GenerateUnseeded?";
        private readonly string _seeded = "/GenerateSeeded?";
        private GeneratorClient()
        {
            _httpClient = new HttpClient
            {
                BaseAddress = _uri
            };
        }

        public static GeneratorClient Instance { get { return Singleton.instance; } }

        private class Singleton
        {
            static Singleton()
            {
            }

            internal static readonly GeneratorClient instance = new GeneratorClient();

        }

        public async Task<NurikabeBoardDto> generateBoards(int height, int width, string difficulty, int timeoutDurationSeconds = 10, int? seed = null)
        {
            var uri = new Uri(FormatUrl(height, width, difficulty, timeoutDurationSeconds, seed));

            HttpResponseMessage response = await _httpClient.GetAsync(uri);

            if (response.IsSuccessStatusCode)
            {
                string content = await response.Content.ReadAsStringAsync();
                return JsonSerializer.Deserialize<NurikabeBoardDto>(content);
            }
            return null;
        }

        private string FormatUrl(int height, int width, string difficulty, int timeoutDurationSeconds, int? seed = null)
        {
            if (seed != null)
            {
                return _seeded + $"difficulty={difficulty}&height={height}&width={width}&seed={seed}&timeoutDurationSeconds={timeoutDurationSeconds}";
            }
            else
            {
                return _unseeded + $"difficulty={difficulty}&height={height}&width={width}&timeoutDurationSeconds={timeoutDurationSeconds}";
            }
        }

    }
}
