﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Web;
using Xamarin.Forms;
using MasterGame.Models.Boards;
using MvvmHelpers;
using System.Windows.Input;

namespace MasterGame.ViewModels
{
    public class IndexSelectionViewModel : BaseViewModel, IQueryAttributable
    {
        private IList<IndexButton> _buttonList;
        public IList<IndexButton> ButtonList
        {
            get => _buttonList;
            private set => SetProperty(ref _buttonList, value);
        }

        private string _difficulty;

        public void ApplyQueryAttributes(IDictionary<string, string> query)
        {
            string difficulty;
            try
            {
                difficulty = HttpUtility.UrlDecode(query["difficulty"]);
            }
            catch (Exception ex)
            {
                Console.Write(ex.Message);
                return;
            }

            _difficulty = difficulty;

            ButtonList = new List<IndexButton>();

            int numBoards = Boards.GetNumBoards(difficulty);

            for (int i = 0; i < numBoards; i++)
            {
                var btn = new IndexButton()
                {
                    Label = $"Level {i+1}",
                    Command = new Command<int>(index => OnIndexSelected(index)),
                    CommandParameter = i
                };
                ButtonList.Add(btn);
            }

            OnPropertyChanged();
        }

        private void OnIndexSelected(int index)
        {
            Shell.Current.GoToAsync($"NurikabeGamePage?difficulty={_difficulty}&newGame=true&index={index}");
        }

        public class IndexButton
        {
            public string Label { get; set; }
            public Command Command { get; set; }
            public int CommandParameter { get; set; }

        }

    }
}
