﻿using MasterGame.Models.Boards;
using MvvmHelpers;
using System;
using System.Collections.Generic;
using System.Web;
using System.Windows.Input;
using Xamarin.Forms;

namespace MasterGame.ViewModels
{

    public class NurikabeGameViewModel : BaseViewModel, IQueryAttributable
    {
        public ICommand BackCommand { get; }
        public string Level { get; private set; }

        NurikabeBoard board;
        public NurikabeBoard Board
        {
            get => board;
            set => SetProperty(ref board, value);
        }

        public void ApplyQueryAttributes(IDictionary<string, string> query)
        {
            bool newGame = false;

            try
            {
                newGame = bool.Parse(HttpUtility.UrlDecode(query["newGame"]));
            }
            catch(Exception ex)
            {
                Console.WriteLine(ex.Message);
            }

            if (!newGame)
            {
                Board = (Application.Current as App).Board;
                Level = (Application.Current as App).Level;
                return;
            }
            
            int? rows = null;
            int? columns = null;
            string difficulty = "easy";
            int index = 0;

            try
            {

                var rowString = HttpUtility.UrlDecode(query["rows"]);
                var columnString = HttpUtility.UrlDecode(query["columns"]);

                rows = Convert.ToInt32(rowString);
                columns = Convert.ToInt32(columnString);
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.Message);
            }

            try
            {
                difficulty = HttpUtility.UrlDecode(query["difficulty"]);
            }
            catch( Exception ex)
            {
                Console.WriteLine(ex.Message);
            }

            if(rows is null || columns is null)
            {
                try
                {
                    var indexString = HttpUtility.UrlDecode(query["index"]);
                    index = Convert.ToInt32(indexString);
                }
                catch(Exception ex)
                {
                    Console.WriteLine(ex.Message);
                }

                Board = new NurikabeBoard(difficulty, 10, (int)index);
                Level = $"Level {index + 1} ({difficulty})";
            }
            else
            {
                Board = new NurikabeBoard(difficulty, (int)rows, (int)columns, 10);
                Level = "";
            }

            (Application.Current as App).AddBoard(Board, Level);
        }

        public NurikabeGameViewModel()
        {
            BackCommand = new Command(OnBackButtonPressed);
        }

        void OnBackButtonPressed()
        {
            Application.Current.MainPage.Navigation.PopToRootAsync();
        }
    }
}
